package sun.net.httpserver;

import java.io.IOException;
import java.io.OutputStream;
import java.io.FilterOutputStream;

class FixedLengthOutputStream extends FilterOutputStream
{
    private long remaining;
    private boolean eof;
    private boolean closed;
    ExchangeImpl t;
    
    FixedLengthOutputStream(ExchangeImpl t, OutputStream out, long remaining) {
        super(out);
        this.eof = false;
        this.closed = false;
        this.t = t;
        this.remaining = remaining;
    }
    
    @Override
    public void write(int n) throws IOException {
        if (this.closed) {
            throw new IOException("stream closed");
        }
        this.eof = (this.remaining == 0L);
        if (this.eof) {
            throw new StreamClosedException();
        }
        this.out.write(n);
        --this.remaining;
        return;
    }
    
    @Override
    public void write(byte[] b, int off, int len) throws IOException {
        if (this.closed) {
            throw new IOException("stream closed");
        }
        this.eof = (this.remaining == 0L);
        if (this.eof) {
            throw new StreamClosedException();
        }
        if ((long)len > this.remaining) {
            throw new IOException("too many bytes to write to stream");
        }
        this.out.write(b, off, len);
        this.remaining -= (long)len;
        return;
    }
    
    @Override
    public void close() throws IOException {
        if (this.closed) {
            return;
        }
        this.closed = true;
        if (this.remaining > 0L) {
            this.t.close();
            throw new IOException("insufficient bytes written to stream");
        }
        this.flush();
        this.eof = true;
        LeftOverInputStream originalInputStream = this.t.getOriginalInputStream();
        if (!originalInputStream.isClosed()) {
            try {
                originalInputStream.close();
            }
            catch (IOException ex) {}
        }
        this.t.getHttpContext().getServerImpl().addEvent(new WriteFinishedEvent(this.t));
        return;
    }
}
